
class Module(object):
    def before_setup(self, properties, arguments, beans):
        raise NotImplementedError()

    def after_setup(self, properties, arguments, context, beans_initializer):
        raise NotImplementedError()



import json
import cherrypy
import inspect
import core
import modules

annotated_controllers = {}


class Module(modules.Module):
    def before_setup(self, properties, arguments, beans):
        pass

    def after_setup(self, properties, arguments, context, beans_initializer):
        controllers_mappings = [core.Mapping(controller(), key) for key, controller in annotated_controllers.iteritems()]

        for m in controllers_mappings:
            beans_initializer.initialize_bean('__controller_'.format(m.bean.__class__.__name__), m.bean, context)
            cherrypy.tree.mount(m.bean, m.path)


def controller(path):
    def wrapped(potential_controller_class):
        annotated_controllers[path] = potential_controller_class
        return potential_controller_class
    return wrapped


class FormEntity(object):
    def __init__(self, entries):
        self.__dict__.update(entries)


class ResponseEntity(object):
    class JsonEncoder(json.JSONEncoder):
        def default(self, o):
            return o.__dict__
    _json_encoder = JsonEncoder()

    def __init__(self, status='ok', data='', error=''):
        self.status = status
        if data != '':
            self.data = data
        if error != '':
            self.error = error

    def tojson(self):
        return ResponseEntity._json_encoder.encode(self)


class FormConstraint(object):
    def __init__(self, name, required='true', type='string', min=0, max=0xffffffff, default=None, nested=None):
        self.name = name
        self.required = required
        self.type = type
        self.min = min
        self.max = max
        self.default = default
        self.nested = nested


class FormBinder(object):
    def __init__(self, form, form_template):
        self.errors = []
        self._form_template = form_template
        self._bind_form(form)

    def has_errors(self):
        return len(self.errors) > 0

    def _bind_form(self, form):
        constraints = [getattr(self._form_template, constraint)
                       for constraint in self._form_template.__dict__
                       if isinstance(getattr(self._form_template, constraint), FormConstraint)]
        fields = {}
        for constraint in constraints:
            value = form.get(constraint.name, None)

            if constraint.nested is not None:
                if value is None and constraint.required:
                    self.errors.append({'field': constraint.name, 'error': 'MISSING_FIELD'})
                    continue
                elif value is not None:
                    another_binder = FormBinder(value, constraint.nested)
                    if another_binder.has_errors():
                        self.errors.extend(another_binder.errors)
                    value = another_binder.entity
            else:
                if value is None and constraint.required:
                    self.errors.append({'field': constraint.name, 'error': 'MISSING_FIELD'})
                    continue
                elif value is None and not constraint.required:
                    value = constraint.default

                if value is not None:
                    if constraint.type == 'int':
                        try:
                            int(value)
                            if value > constraint.max:
                                self.errors.append({'field': constraint.name, 'error': 'MAX'})
                            if value < constraint.min:
                                self.errors.append({'field': constraint.name, 'error': 'MIN'})
                        except ValueError:
                            self.errors.append({'field': constraint.name, 'error': 'BAD_TYPE'})
                    elif constraint.type == 'float':
                        try:
                            float(value)
                            if value > constraint.max:
                                self.errors.append({'field': constraint.name, 'error': 'MAX'})
                            if value < constraint.min:
                                self.errors.append({'field': constraint.name, 'error': 'MIN'})
                        except ValueError:
                            self.errors.append({'field': constraint.name, 'error': 'BAD_TYPE'})
                    elif constraint.type == 'string':
                        try:
                            str(value)
                            if len(value) > constraint.max:
                                self.errors.append({'field': constraint.name, 'error': 'MAX_LENGTH'})
                            if len(value) < constraint.min:
                                self.errors.append({'field': constraint.name, 'error': 'MIN_LENGTH'})
                        except ValueError:
                            self.errors.append({'field': constraint.name, 'error': 'BAD_TYPE'})
            fields[constraint.name] = value
        self.entity = FormEntity(fields)


def form(form_template):
    def form_retriever(func):
        def wrapped(instance, *args, **kwargs):
            try:
                form_json = json.load(cherrypy.request.body)
            except ValueError:
                return ResponseEntity(status='error', error='Cannot parse request body')

            form_binder = FormBinder(form_json, form_template)
            kwargs.update({'form': form_binder})
            return func(instance, *args, **kwargs)
        return wrapped
    return form_retriever


def content_type(mime):
    def value_retriever(func):
        def wrapped(instance, *args, **kwargs):
            cherrypy.response.headers['Content-Type'] = mime
            return func(instance, *args, **kwargs)
        return wrapped
    return value_retriever


def exception_handler():
    cherrypy.response.status = 500
    cherrypy.response.headers['Content-Type'] = 'application/json'
    cherrypy.response.body = ResponseEntity(status='error', error='Internal error').tojson()


class BaseController(core.BaseBean):
    _cp_config = {'request.error_response': exception_handler,
                  'show_tracebacks': False,
                  'show_mismatched_params': False,
                  'throw_errors': False}

    def _set_bad_request_error(self):
        cherrypy.response.status = 400
        cherrypy.response.headers['Content-Type'] = 'application/json'

    @cherrypy.expose
    def default(self, *vpath, **params):
        def find_handling_method():
            return getattr(self, cherrypy.request.method, None)

        def setup_default_response_header():
            cherrypy.response.headers['Content-Type'] = 'application/json'

        def dispatch_event(method):
            if not method:
                cherrypy.response.status = 404
                return ResponseEntity(status='error', error="Not found")
            method_args = inspect.getargspec(method)
            if len(method_args.args) > 1 or method_args.varargs or method_args.keywords:
                result = method(*vpath, **params)
            else:
                result = method()
            return result

        try:
            setup_default_response_header()
            method = find_handling_method()
            result = dispatch_event(method)

            if cherrypy.response.headers['Content-Type'] == 'application/json':
                if result.status == 'error' and cherrypy.response.status != 404:
                    self._set_bad_request_error()
                return result.tojson()
            else:
                return result
        except ValueError:
            self._set_bad_request_error()
            return ResponseEntity(status='error', error='Bad Request').tojson()



import requests
import cherrypy


def load_remote_configuration(properties):
    remote_config_url = properties.get('remote_config_url')
    app_name = properties.get('app_name')

    if remote_config_url is None or not remote_config_url:
        cherrypy.log.error('Remote config URL not present. Skipping.')
        return properties
    if app_name is None or not app_name:
        cherrypy.log.error('Remote config URL is present but applciation name was not specified!')
        return properties

    if not remote_config_url.endswith('/'):
        remote_config_url += '/'

    remote_properties = {}
    try:
        config_server_response = requests.get(remote_config_url + app_name + '/default')
        if config_server_response.status_code != 200:
            cherrypy.log.error('Error loading remote properties! Status code ' + str(config_server_response.status_code))
            return properties

        sources = [source['source'] for source in config_server_response.json()['propertySources']]
        for source in sources:
            remote_properties.update(source)
    except requests.exceptions.ConnectionError:
        cherrypy.log.error('Error while connecting to remote config server!')
        return properties

    new_properties = properties.copy()
    new_properties.update(remote_properties)
    return new_properties



import web
import db
import core
import testing
import beans
import client
from core import log, scan_packages
from web import controller
from beans import bean

__all__ = ['beans', 'core', 'web', 'db', 'testing', 'client', 'log', 'controller', 'bean', 'scan_packages']



import cherrypy
import logging
from logging import handlers

DEFAULT_APPLICATION_LOG = 'log/application.log'
DEFAULT_ACCESS_LOG = 'log/access.log'
DEFAULT_MAX_LOGS = 10
DEFAULT_MAX_LOG_SIZE = 10
MB_SIZE = 1000000
FORMATTER = logging.Formatter('[%(asctime)s] %(levelname)s %(filename)s:%(funcName)s: %(message)s')


def setup_logging(logger, properties):
    system_log = cherrypy.log
    system_log.error_file = ""
    system_log.access_file = ""

    max_bytes = getattr(system_log, "rot_maxBytes", properties.get('logs_max_size', DEFAULT_MAX_LOG_SIZE) * MB_SIZE)
    backup_count = getattr(system_log, "rot_backupCount", properties.get('logs_max_files', DEFAULT_MAX_LOGS))

    error_log_file = getattr(system_log, "rot_error_file",
                             properties.get('logs_application_file', DEFAULT_APPLICATION_LOG))
    access_log_file = getattr(system_log, "rot_access_file", properties.get('logs_access_file', DEFAULT_ACCESS_LOG))

    application_log_handler = _setup_application_log(backup_count, error_log_file, max_bytes)
    access_log_handler = _setup_access_log_handler(access_log_file, backup_count, max_bytes)

    logger.addHandler(application_log_handler)
    logger.addHandler(logging.StreamHandler())

    system_log.error_log.addHandler(application_log_handler)
    system_log.access_log.handlers = []
    system_log.access_log.addHandler(access_log_handler)


def _setup_access_log_handler(access_log_file, backup_count, max_bytes):
    access_log_handler = handlers.RotatingFileHandler(access_log_file, 'a', max_bytes, backup_count)
    access_log_handler.setLevel(logging.INFO)
    access_log_handler.setFormatter(cherrypy._cplogging.logfmt)
    return access_log_handler


def _setup_application_log(backup_count, error_log_file, max_bytes):
    application_log_handler = handlers.RotatingFileHandler(error_log_file, 'a', max_bytes, backup_count)
    application_log_handler.setLevel(logging.INFO)
    application_log_handler.setFormatter(FORMATTER)
    return application_log_handler



import ConfigParser
import collections
import logs
import logging
import beans
import cherrypy
import pkgutil
import web
import remote_configuration


DEFAULT_HOST = '127.0.0.1'
DEFAULT_PORT = 8000
Mapping = collections.namedtuple('Mapping', ['bean', 'path'])

log = logging.getLogger('framepy_logger')


class Context(object):
    def __init__(self, entries):
        self.__dict__.update(entries)


class BaseBean(object):
    def __init__(self):
        self.context = None

    def initialize(self, context):
        self.context = context


def _update_config(load_properties):
    cherrypy.config.update({'server.socket_port': load_properties.get('server_port', DEFAULT_PORT),
                            'server.socket_host': load_properties.get('server_host', DEFAULT_HOST)})


def _load_properties(file):
    parser = ConfigParser.RawConfigParser()
    try:
        parser.readfp(open(file, 'r'))
    except IOError:
        cherrypy.log.error('Cannot open properties file {0}'.format(file))
        raise IOError('Cannot open properties file {0}'.format(file))
    return {key: value for (key, value) in parser.items('Properties')}


def _create_context(loaded_properties, modules, kwargs):
    beans = {}
    for module in modules:
        module.before_setup(loaded_properties, kwargs, beans)

    return Context(beans)


def _after_setup(context, modules, kwargs, properties, beans_initializer):
    beans_initializer.initialize_all(context)
    for module in modules:
        module.after_setup(properties, kwargs, context, beans_initializer)


def scan_packages():
    for modname in (modname for importer, modname, ispkg in pkgutil.walk_packages('.') if '.' in modname):
        __import__(modname)


def init_context(properties,
                 modules=(),
                 **kwargs):
    beans_initializer = beans.BeansInitializer()
    modules = (web.Module(),) + modules

    loaded_properties = _load_properties(properties)
    loaded_properties = remote_configuration.load_remote_configuration(loaded_properties)
    _update_config(loaded_properties)
    logs.setup_logging(log, loaded_properties)
    context = _create_context(loaded_properties, modules, kwargs)

    _after_setup(context, modules, kwargs, loaded_properties, beans_initializer)

    return cherrypy.tree


def start_standalone_application():
    cherrypy.engine.start()
    cherrypy.engine.block()
